@@ -1,35 +1,37 @@
 import typing
 from dataclasses import dataclass, field
 
-from .. import constants, errors, utils
+from .. import constants, errors, typedefs, utils
 
 
 __all__ = (
     'JsonRpcResponse',
     'JsonRpcBatchResponse',
-    'UnlinkedResults',
-    'DuplicatedResults',
+    'JsonRpcUnlinkedResults',
+    'JsonRpcDuplicatedResults',
 )
 
 
 @dataclass
 class JsonRpcResponse:
+    id: typing.Optional[typedefs.JsonRpcIdType] = None
     jsonrpc: str = constants.VERSION_2_0
-    id: typing.Any = constants.NOTHING
-    result: typing.Any = constants.NOTHING
+    result: typing.Any = None
     error: typing.Optional[errors.JsonRpcError] = None
-    context: dict = field(default_factory=dict)
+    context: typing.MutableMapping = field(default_factory=dict)
 
     @property
     def is_notification(self) -> bool:
-        return self.id in constants.EMPTY_VALUES
+        return self.id is None
 
     @classmethod
-    def from_dict(cls, data: dict, *, error_map: typing.Optional[dict] = None, **kwargs) -> 'JsonRpcResponse':
+    def load(cls,
+             data: typing.Any, *,
+             error_map: typing.Optional[typing.Mapping] = None, **kwargs) -> 'JsonRpcResponse':
         cls._validate_json_response(data)
 
         response = cls(
-            id=data.get('id', constants.NOTHING),
+            id=data.get('id'),
             jsonrpc=data.get('jsonrpc', constants.VERSION_2_0),
             result=data.get('result'),
             **kwargs,
@@ -40,15 +42,13 @@ class JsonRpcResponse:
 
         return response
 
-    def to_dict(self) -> typing.Optional[dict]:
-        data = {'jsonrpc': self.jsonrpc}
+    def dump(self) -> typing.Mapping[str, typing.Any]:
+        data: typing.Dict[str, typing.Any] = {
+            'id': self.id,
+            'jsonrpc': self.jsonrpc,
+        }
 
-        if self.id in constants.EMPTY_VALUES:
-            data['id'] = None
-        else:
-            data['id'] = self.id
-
-        if self.error in constants.EMPTY_VALUES:
+        if self.error is None:
             data['result'] = self.result
         else:
             data['error'] = {'code': self.error.code, 'message': self.error.message}
@@ -60,7 +60,7 @@ class JsonRpcResponse:
 
     @staticmethod
     def _validate_json_response(data: typing.Any) -> None:
-        if not isinstance(data, dict):
+        if not isinstance(data, typing.Mapping):
             raise errors.InvalidRequest
 
         utils.validate_jsonrpc(data.get('jsonrpc'))
@@ -69,11 +69,13 @@ class JsonRpcResponse:
             raise errors.InvalidRequest('"result" or "error" not found in data.', data={'raw_response': data})
 
     @staticmethod
-    def _add_error(response: 'JsonRpcResponse', error: typing.Any, *, error_map: typing.Optional[dict] = None) -> None:
-        if not isinstance(error, dict):
+    def _add_error(response: 'JsonRpcResponse',
+                   error: typing.Any, *,
+                   error_map: typing.Optional[typing.Mapping] = None) -> None:
+        if not isinstance(error, typing.Mapping):
             raise errors.InvalidRequest
 
-        if not ({'code', 'message'}) <= error.keys():
+        if not {'code', 'message'} <= error.keys():
             raise errors.InvalidRequest
 
         if error_map:
@@ -90,44 +92,42 @@ class JsonRpcResponse:
 
 @dataclass
 class JsonRpcBatchResponse:
-    responses: typing.List[JsonRpcResponse] = field(default_factory=list)
-
-    def to_list(self) -> typing.List[dict]:
-        return [response.to_dict() for response in self.responses]
+    responses: typing.Tuple[JsonRpcResponse, ...] = field(default_factory=tuple)
 
     @classmethod
-    def from_list(cls, data: list, *, error_map: typing.Optional[dict] = None, **kwargs) -> 'JsonRpcBatchResponse':
-        responses = [
-            JsonRpcResponse.from_dict(item, error_map=error_map, **kwargs)
+    def load(cls,
+             data: typing.Any, *,
+             error_map: typing.Optional[typing.Mapping] = None,
+             **kwargs) -> 'JsonRpcBatchResponse':
+        if not isinstance(data, typing.Sequence):
+            raise errors.InvalidRequest('A batch request must be of the list type.')
+
+        return cls(responses=tuple(
+            JsonRpcResponse.load(item, error_map=error_map, **kwargs)
             for item in data
-        ]
+        ))
 
-        return cls(responses=responses)
+    def dump(self) -> typing.Tuple[typing.Mapping[str, typing.Any], ...]:
+        return tuple(response.dump() for response in self.responses)
 
 
 @dataclass
-class UnlinkedResults:
-    data: list = field(default_factory=list)
+class JsonRpcUnlinkedResults:
+    results: typing.MutableSequence = field(default_factory=list)
 
     def __bool__(self) -> bool:
-        return len(self.data) > 0
-
-    def get(self) -> list:
-        return self.data
+        return len(self.results) > 0
 
     def add(self, value: typing.Any) -> None:
-        self.data.append(value)
+        self.results.append(value)
 
 
 @dataclass
-class DuplicatedResults:
-    data: list = field(default_factory=list)
+class JsonRpcDuplicatedResults:
+    results: typing.MutableSequence = field(default_factory=list)
 
     def __bool__(self) -> bool:
-        return len(self.data) > 0
-
-    def get(self) -> list:
-        return self.data
+        return len(self.results) > 0
 
     def add(self, value: typing.Any) -> None:
-        self.data.append(value)
+        self.results.append(value)
